#include "TePDIEMClas.hpp"

#include "TePDIMathFunctions.hpp"

#include "TePDIStrategyFactory.hpp"

#include "TePDIAlgorithmFactory.hpp"

#include "TePDIUtils.hpp"



#include <TeBox.h>

#include <TeUtils.h>

#include <TeRasterParams.h>

#include <TeRaster.h>

#include <TeAgnostic.h>



void TePDIEMClas::ResetState( const TePDIParameters& )

{



}



TePDIEMClas::TePDIEMClas()

{

};



TePDIEMClas::~TePDIEMClas()

{

};



bool TePDIEMClas::CheckParameters(const TePDIParameters& parameters) const

{

  // Checking input_rasters

  TePDITypes::TePDIRasterVectorType input_rasters;

  TEAGN_TRUE_OR_RETURN(parameters.GetParameter("input_rasters", input_rasters), 

    "Missing parameter: input_rasters");

  TEAGN_TRUE_OR_RETURN(input_rasters.size() > 0, 

    "Invalid input rasters number");



  vector<int> bands;

  TEAGN_TRUE_OR_RETURN(parameters.GetParameter("bands", bands), "Missing parameter: bands");

  TEAGN_TRUE_OR_RETURN(bands.size() == input_rasters.size(), Te2String(input_rasters.size()) + " Invalid parameter: bands number " + Te2String(bands.size()));



  for(unsigned int input_rasters_index = 0 ; input_rasters_index < input_rasters.size(); input_rasters_index++)

  {

    TEAGN_TRUE_OR_RETURN(input_rasters[input_rasters_index].isActive(), "Invalid parameter: input_raster " + Te2String(input_rasters_index) + " inactive");

    TEAGN_TRUE_OR_RETURN(input_rasters[input_rasters_index]->params().status_ != TeRasterParams::TeNotReady, "Invalid parameter: input_raster " + Te2String(input_rasters_index) + " not ready");

    TEAGN_TRUE_OR_RETURN(input_rasters[input_rasters_index]->params().nlines_ == input_rasters[0]->params().nlines_, "Invalid parameter: input_raster " + Te2String(input_rasters_index) + " with imcompatible number of lines");

    TEAGN_TRUE_OR_RETURN(input_rasters[input_rasters_index]->params().ncols_ == input_rasters[0]->params().ncols_, "Invalid parameter: input_raster " + Te2String(input_rasters_index) + " with imcompatible number of columns");

  }



  int M;

  TEAGN_TRUE_OR_RETURN(parameters.GetParameter("classes_to_find", M), "Missing parameter: classes_to_find");



  // Checking output_raster

  TePDITypes::TePDIRasterPtrType output_raster;

  TEAGN_TRUE_OR_RETURN(parameters.GetParameter("output_raster", output_raster), "Missing parameter: output_raster");

  TEAGN_TRUE_OR_RETURN(output_raster.isActive(), "Invalid parameter: output_raster inactive");

  TEAGN_TRUE_OR_RETURN(output_raster->params().status_ != TeRasterParams::TeNotReady, "Invalid parameter: output_raster not ready");

  return true;

}



bool TePDIEMClas::RunImplementation()

{

  // Setting the parameters

  TePDITypes::TePDIRasterVectorType input_rasters;

  params_.GetParameter("input_rasters", input_rasters);



  vector<int> bands;

  params_.GetParameter("bands", bands);



  W = input_rasters[0]->params().ncols_;

  H = input_rasters[0]->params().nlines_;

  N = 0;

  TOTAL_ATTRIBUTES = bands.size();

  params_.GetParameter("classes_to_find", M);

  if (!params_.GetParameter("sx", SX))

    SX = 1;

  if (!params_.GetParameter("sy", SY))

    SY = 1;

  if (!params_.GetParameter("max_iterations", MAX_ITERATIONS))

    MAX_ITERATIONS = 3;

  if (!params_.GetParameter("epsilon", EPSILON))

    EPSILON = 15.0;

  for (int i = 0; i < W; i+=SX)

    for (int j = 0; j < H; j+=SY)

      N++;

  double  START_COVAR = 1000.0,

      *variations = (double *) malloc(M * sizeof(double));



  // Start the algorithm

  vector<TeMatrix>  covars, // covariance matrix

            old_covars, // to prevent problems in inverse function

            means, // mean vectors

            old_means, // to compare difference during the E-M process

            X, // image points

            original_X; // original image points (hole set)

  vector<double>  P; // Probability for each cluster

  double  **P_C_x = (double **) malloc(M * sizeof(double)), // Probability a posteriori

      **original_P_C_x = (double **) malloc(M * sizeof(double));



  // Initializing data...

  for (int i = 0; i < M; i++)

  {

    P_C_x[i] = (double *) malloc(N * sizeof(double));

    original_P_C_x[i] = (double *) malloc(W * H * sizeof(double));

    for (int j = 0; j < N; j++)

      P_C_x[i][j] = 0.0;

    for (int j = 0; j < W * H; j++)

      original_P_C_x[i][j] = 0.0;

  }

  // initialize covariances, means and P

  TeMatrix tmp_covar;

  tmp_covar.Init(TOTAL_ATTRIBUTES, START_COVAR);

  TeMatrix tmp_mean;

  unsigned seed = time(0);

  srand(seed);

//   double initial_mean = 255 / (M - 1);

    for (int j = 0; j < M; j++)

  {

    double *random_means = (double *)malloc(TOTAL_ATTRIBUTES * sizeof(double));

    for (int r = 0; r < TOTAL_ATTRIBUTES; r++)

    {

      random_means[r] = rand() % 255;

//       random_means[r] = initial_mean * j;

    }

    tmp_mean.Init(TOTAL_ATTRIBUTES, 1, random_means);

    means.push_back(tmp_mean);

    old_means.push_back(tmp_mean);

    covars.push_back(tmp_covar);

    old_covars.push_back(tmp_covar);

    P.push_back((double) 1 / M);

  }



  // Generates the attributes vector

  TeMatrix tmp_X;



  double *tmp = (double*) malloc(TOTAL_ATTRIBUTES * sizeof(double));

  for (int i = 0; i < W; i+=SX)

    for (int j = 0; j < H; j+=SY)

    {

      for (int b = 0; b < TOTAL_ATTRIBUTES; b++)

        input_rasters[b]->getElement(i, j, tmp[b]);

      tmp_X.Init(TOTAL_ATTRIBUTES, 1, tmp);

      X.push_back(tmp_X);

    }



  // Store the hole set of pixels

  for (int i = 0; i < W; i++)

    for (int j = 0; j < H; j++)

    {

      for (int b = 0; b < TOTAL_ATTRIBUTES; b++)

        input_rasters[b]->getElement(i, j, tmp[b]);

      tmp_X.Init(TOTAL_ATTRIBUTES, 1, tmp);

      original_X.push_back(tmp_X);

    }



  // Start to estimate the data

  int num_iterations = 0;

  bool stop_condition = false;

  TeMatrix  tmp_matrix,

        tmp_inv,

        tmp_trans,

        tmp_prod;



  while (stop_condition == false)

  {

    // Calculate P_C_x

    for (int j = 0; j < M; j++)

    {

      for (int k = 0; k < N; k++)

      {

        double denom = 0.0;

        for (int k1 = 0; k1 < M; k1++)

        {

          tmp_matrix = X[k] - means[k1];

          if (!covars[k1].Inverse(tmp_inv))

          {

            covars[k1].Init(TOTAL_ATTRIBUTES, START_COVAR);

            covars[k1].Inverse(tmp_inv);

          }

          tmp_matrix.Transpose(tmp_trans);

          tmp_prod.Init();

          tmp_prod = tmp_trans * tmp_inv * tmp_matrix;



          denom += pow(covars[k1].Determinant(), -0.5) * exp(-0.5 * tmp_prod(0, 0)) * P[k1];

        }



        tmp_matrix = X[k] - means[j];





        if (!covars[j].Inverse(tmp_inv))

        {

          covars[j].Init(TOTAL_ATTRIBUTES, START_COVAR);

          covars[j].Inverse(tmp_inv);

        }



        tmp_matrix.Transpose(tmp_trans);



        tmp_prod = tmp_trans * tmp_inv * tmp_matrix;



        P_C_x[j][k] = pow(covars[j].Determinant(), -0.5) * exp(-0.5 * tmp_prod(0,0)) * P[j] / denom;

      }

    }



    // Update means, covariances and Probabilities

    for (int j = 0; j < M; j++)

    {

      double tmp_denom = 0.0;

      TeMatrix  tmp_num_mean,

            tmp_num_covar,

            tmp_sub;

      tmp_num_mean.Init(TOTAL_ATTRIBUTES, 1, 0.0);

      tmp_num_covar.Init(TOTAL_ATTRIBUTES, 0.0);



      for (int k = 1; k < N; k++)

      {

        // sum for mean numerator

        tmp_num_mean = tmp_num_mean + P_C_x[j][k] * X[k];

        // sum for both denominators

        tmp_denom += P_C_x[j][k];

        // sum for covar numerator

        tmp_matrix = X[k] - means[j];

        tmp_matrix.Transpose(tmp_trans);

        tmp_prod = tmp_matrix * tmp_trans;

        tmp_num_covar = tmp_num_covar + P_C_x[j][k] * tmp_prod;

      }



      means[j] =  (1 / tmp_denom) * tmp_num_mean;

       covars[j] = (1 / tmp_denom) * tmp_num_covar;

      P[j] = tmp_denom / N;



      tmp_sub = means[j] - old_means[j];

      tmp_sub.Transpose(tmp_trans);

      tmp_prod = tmp_trans * tmp_sub;

      variations[j] = sqrt(tmp_prod(0, 0));

      means[j].Print();

      covars[j].Print();

    }



    TeMatrix  tmp_sub,

          shift;



    shift.Init(TOTAL_ATTRIBUTES, 1, 255.0);

    double shift_threshold = 30.0;

    for (int j = 0; j < M; j++)

      for (int j1 = (j + 1); j1 < M; j1++)

      {

        tmp_sub = means[j] - means[j1];

        tmp_sub.Transpose(tmp_trans);

        tmp_prod = tmp_trans * tmp_sub;

        if (sqrt(tmp_prod(0, 0)) < shift_threshold)

        {

          means[j1] = shift - means[j1];

        }

      }



    double variation = 0.0;

    for (int j = 0; j < M; j++)

      variation += variations[j];

    variation = sqrt(variation);





    old_means = means;

    old_covars = covars;

    if (++num_iterations >= MAX_ITERATIONS)

      stop_condition = true;

    if (variation < EPSILON)

      stop_condition = true;

  }



  // Classifying input image



  double *class_covars_determinant = (double *) malloc(M * sizeof(double));

  vector<TeMatrix> class_covars_inverse;

  for (int j = 0; j < M; j++)

  {

    class_covars_determinant[j] = covars[j].Determinant();

    covars[j].Inverse(tmp_inv);

    class_covars_inverse.push_back(tmp_inv);

  }

  for (int j = 0; j < M; j++)

  {

    for (int k = 0; k < H * W; k++)

    {

      double denom = 0.0;

      for (int k1 = 0; k1 < M; k1++)

      {

        tmp_matrix = original_X[k] - means[k1];



        tmp_matrix.Transpose(tmp_trans);

        tmp_prod = tmp_trans * class_covars_inverse[k1] * tmp_matrix;



        denom += pow(class_covars_determinant[k1], -0.5) * exp(-0.5 * tmp_prod(0, 0)) * P[k1];

      }

      tmp_matrix = original_X[k] - means[j];

      tmp_matrix.Transpose(tmp_trans);

      tmp_prod = tmp_trans * class_covars_inverse[j] * tmp_matrix;



      original_P_C_x[j][k] = pow(class_covars_determinant[j], -0.5) * exp(-0.5 * tmp_prod(0,0)) * P[j] / denom;

    }

  }



  // Plotting results

  // Initializating the output_raster

  TePDITypes::TePDIRasterPtrType output_raster;

  params_.GetParameter("output_raster", output_raster);

  TeRasterParams output_raster_params = output_raster->params();

  output_raster_params.nBands(3);

  output_raster_params.setDataType(TeINTEGER, -1);

  output_raster_params.setNLinesNColumns( H, W );

  output_raster_params.setPhotometric(TeRasterParams::TeRGB, -1);

  TEAGN_TRUE_OR_RETURN(output_raster->init(output_raster_params), 

    "Output raster reset error" );



  vector<int> colors_R, colors_G, colors_B;

  colors_R.push_back(0); colors_G.push_back(0); colors_B.push_back(255);

  colors_R.push_back(0); colors_G.push_back(255); colors_B.push_back(0);

  colors_R.push_back(255); colors_G.push_back(0); colors_B.push_back(0);

  colors_R.push_back(0); colors_G.push_back(255); colors_B.push_back(255);

  colors_R.push_back(255); colors_G.push_back(0); colors_B.push_back(255);

  colors_R.push_back(255); colors_G.push_back(255); colors_B.push_back(0);

  colors_R.push_back(255); colors_G.push_back(255); colors_B.push_back(255);

  colors_R.push_back(0); colors_G.push_back(0); colors_B.push_back(0);



  int k = 0;

  for (int c = 0; c < W; c++)

    for (int l = 0; l < H; l++)

    {

      int  max = 0,

        R, G, B;

      for (int j = 0; j < M; j++)

        if (original_P_C_x[j][k] > original_P_C_x[max][k])

          max = j;

      R = colors_R[max],

      G = colors_G[max],

      B = colors_B[max];

      output_raster->setElement(c, l, R, G, B);

      k++;

    }



  return true;

}

