#ifndef TEPDIMIXMODEL_HPP
#define TEPDIMIXMODEL_HPP

#include <TePDIStrategyFactory.hpp>
#include <TeAgnostic.h>
#include <TePDIAlgorithm.hpp>
#include <TePDIParameters.hpp>
#include <TeSharedPtr.h>

/**
* @brief This is the base class for image mixmodel algorithms.
* @author Felipe Castro da Silva <felipe@dpi.inpe.br>
* @ingroup PDIAlgorithms
*
* @note The general required parameters:
* @param mixmodel_type (std::string) - MixModel Strategy type
*
* @note The specific parameters: See each mixmodel strategy for reference.   
*/

class TePDIMixModel : public TePDIAlgorithm
{
	
	public :
		
/*		typedef TeSharedPtr< TePDIMixModel > pointer;
		typedef const TeSharedPtr< TePDIMixModel > const_pointer;*/
		
		/**
		* Default Constructor.
		*
		*/
		TePDIMixModel();
		
		/**
		* Default Destructor
		*/
		virtual ~TePDIMixModel();
		
		/**
		* Checks if the supplied parameters fits the requirements of each
		* PDI algorithm implementation.
		*
		* Error log messages will be generated. No exceptions generated.
		*
		* @param parameters The parameters to be checked.
		* @return true if the parameters are OK. false if not.
		*/
		bool CheckParameters( const TePDIParameters& parameters ) const;

	protected :
	
		/**
		* Reset the internal state to the initial state.
		*
		* @param params The new parameters referente at initial state.
		*/
		void ResetState( const TePDIParameters& params );

		/**
		* Runs the current algorithm implementation.
		*
		* @return true if OK. false on error.
		*/
		bool RunImplementation();

};
  
/** @example TePDIMixModel_test.cpp
 *    Shows how to use this class.
 */

#endif
