#include <string>
#include <map>

using namespace std;

/**
* @brief This is the class contains the component information for mixmodel algorithms.
* @author Felipe Castro da Silva <felipe@dpi.inpe.br>
* @ingroup PDIMixModel
*/

class TePDIMixModelComponent
{
	public:
/**
* Default constructor.
* @param l name of the component (e.g. cloud, vegetation, ...).
*/
		TePDIMixModelComponent(string l);
/**Default destructor.*/
		~TePDIMixModelComponent();

/**
* Insert pixel.
* @param bn band number where the pixel will be stored.
* @param p pixel value.
* @return true if pixel was inserted, false if not.
*/
		bool insertPixel(unsigned int bn, double p);
/**
* Remove pixel.
* @param bn band number of the pixel to be removed.
* @return true if pixel was removed, false if not.
*/
		bool removePixel(unsigned int bn);
/**
* Return pixel value.
* @param bn pixel band number.
* @return if bn exists, pixel value, if note -1.0.
*/
		double getPixel(unsigned int bn);
/**
* Return bands number.
* @return bands number in the component.
*/
		unsigned getSize();
/**Clear all the component.*/
		void clearPixels();

	private:
/**The component name (e.g. cloud, vegetation, ...).*/
		string label;
/**Store band number and pixel value.*/
		map<unsigned int, double> pixels;
};
