#include <string>

using namespace std;

/**
* @brief This is the class contains the spectral band information for mixmodel algorithms.
* @author Felipe Castro da Silva <felipe@dpi.inpe.br>
* @ingroup PDIMixModel
*/

class TePDIMixModelSpectralBand
{
	public:
/**
* Default constructor.
* @param s sensor number.
* @param b band number.
* @param ll lower spectral range bound.
* @param ul upper spectral range bound.
* @note In "princo" strategy case, the parameters s, ll and ul, can be zero.
* @param l spectral band name (e.g. CBERS_RED, CBERS_BLUE, CBERS_GREEN, ...).
*/
		TePDIMixModelSpectralBand(unsigned int s, unsigned int b, double ll, double ul, string l);
/**Default constructor.*/
		~TePDIMixModelSpectralBand();
/**
* Return the sensor.
* @return sensor number.
*/
		unsigned int getSensor();
/**
* Return band number.
* @return band number.
*/
		unsigned int getBand();
/**
* Return lower spectral range bound.
* @return lower spectral range bound.
*/
		double getLowerLimit();
/**
* Return upper spectral range bound.
* @return upper spectral range bound.
*/
		double getUpperLimit();
/**
* Return spectral band name.
* @return spectral band name.
*/
		string getLabel();

	private:
/**Store sensor number, band number, lower spectral range bound, upper spectral range bound and spectral band name*/	
		unsigned int	sensor,
				band;
		double	lowerlimit,
			upperlimit;
		string label;

};
