#ifndef TEPDIMIXMODELSPECTRALBANDLIST_HPP
#define TEPDIMIXMODELSPECTRALBANDLIST_HPP

#include <string>
#include <vector>

#include <TeAgnostic.h>
#include <TePDIMixModelSpectralBand.hpp>

using namespace std;

/**
* @brief This is the class contains the list of spectral bands for mixmodel algorithms.
* @author Felipe Castro da Silva <felipe@dpi.inpe.br>
* @ingroup PDIMixModel
*/

class TePDIMixModelSpectralBandList
{
	public:
/**Default constructor.*/
		TePDIMixModelSpectralBandList();
/**Default destructor.*/
		~TePDIMixModelSpectralBandList();
/**
* Insert spectral band.
* @param sb spectral band to be stored.
* @return true if the spectral band was inserted, false if not.
*/
		bool insertSpectralBand(TePDIMixModelSpectralBand sb);
/**
* Load spectral band file.
* @note This file have the following format: sensor_number band_number lower_spectral_range_bound upper_spectral_range_bound spectral_band_name (e.g. 160 1 0.450000 0.520000 Landsat5-TM1)
* @param filename file name with spectral bands information.
* @return true if the file was loaded sucessfully, false if not.
*/
		bool loadSpectralBandsFromFile(string filename);
/**
* Return spectral band list size.
* @return return the spectralbandlist size.
*/
		unsigned int getSize();
/**
* Return spectral band name.
* @param b band number.
* @return return the spectral band name.
*/
		string getSpectralBandLabel(unsigned int b);
/**
* Return spectral band.
* @param b band number.
* @return return the spectral band.
*/
		TePDIMixModelSpectralBand getSpectralBand(unsigned int b);
	private:
/**Store the spectral bands*/
		vector<TePDIMixModelSpectralBand> bands;
};
#endif
