#include "TePDIPIManager.hpp"

#include <TeAgnostic.h>

#include <TeProgress.h>
#include <math.h>


TePDIPIManager::TePDIPIManager( const std::string& message, 
  unsigned long int tsteps, bool enabled )
{
  my_settings_.curr_manager_ptr_ = this;
  Reset( message, tsteps );
  Toggle( enabled );
}


TePDIPIManager::~TePDIPIManager()
{
  Toggle( false );
}


void TePDIPIManager::Toggle( bool enabled )
{
  TePDIPIManagerSettings& global_settings = GetGlobalSettings();
  
  if( ( ! my_settings_.enabled_ ) && enabled ) {
    old_global_settings_ = global_settings;
    my_settings_.enabled_ = true;
    global_settings = my_settings_;
    
    UpdateProgressInterface( global_settings );
  } if( my_settings_.enabled_ && ( ! enabled ) ) {
    global_settings = old_global_settings_;
    my_settings_.enabled_ = false;
    
    if( global_settings.curr_manager_ptr_ == 0 ) {
      TeProgressBase* prog_intef = TeProgress::instance();
      
      if( prog_intef ) {
        prog_intef->reset();
      }
    } else {    
      global_settings.curr_manager_ptr_->UpdateProgressInterface( 
        global_settings );
    }
  }
}


bool TePDIPIManager::UpdateProgressInterface( 
  const TePDIPIManagerSettings& settings ) const
{
  if( settings.curr_manager_ptr_ == this ) {
    TeProgressBase* prog_intef = TeProgress::instance();
    
    if( prog_intef ) {
      if( prog_intef->wasCancelled() ) {
        return true;
      } else {
        if( settings.enabled_ ) {
          prog_intef->setMessage( settings.message_ );
          prog_intef->setTotalSteps( 100 );
          prog_intef->setProgress( settings.propcurrstep_ );
        } else {
          prog_intef->reset();
        } 
       
        return false; 
      }
    } else {
      return false;
    }
  } else {
    return false;
  }
}


bool TePDIPIManager::Update( unsigned long int step )
{
  if( step > my_settings_.tsteps_ ) {
    return false;
  }

  my_settings_.currstep_ = step;
  
  int propcurrstep = 
    (int)( 100. * ( ( (double)step ) / 
    ( (double)my_settings_.tsteps_ ) ) );  
  
  if( my_settings_.enabled_ ) {
    if( propcurrstep != my_settings_.propcurrstep_ ) {
      my_settings_.propcurrstep_ = propcurrstep;
      return UpdateProgressInterface( my_settings_ );
    } else {
      return false;
    }
  } else {
    return false;
  }
}


bool TePDIPIManager::Increment()
{
  return Update( my_settings_.currstep_ + 1 );
}


void TePDIPIManager::Reset( const std::string& message, 
  unsigned long int tsteps )
{
  my_settings_.tsteps_ = tsteps; 
  my_settings_.currstep_ = 0;
  my_settings_.message_ = message;

  if( my_settings_.enabled_ ) {
    UpdateProgressInterface( my_settings_ );
  }  
}

