#include "TeThreadLock.h"

#include "TeAgnostic.h"

#if TePLATFORM == TePLATFORMCODE_MSWINDOWS

  #include  <stdio.h>

#elif TePLATFORM == TePLATFORMCODE_LINUX
    
#else
    #error "Unsuported plataform"
#endif   

TeThreadLock::TeThreadLock()
{
  #if TePLATFORM == TePLATFORMCODE_MSWINDOWS
  
    m_sa_.nLength = sizeof( m_sa_ );
    m_sa_.lpSecurityDescriptor = NULL;
    m_sa_.bInheritHandle = TRUE;
/*    
    char szUUID[ 41 ] = { 0 };
    SYSTEMTIME st;
    ::GetSystemTime( &st );
    sprintf( szUUID, "UUID:%02d:%02d:%02d:%03d:%02d:%02d:%02d:%04d", 
      st.wDay, st.wDayOfWeek, st.wHour, st.wMilliseconds, st.wMinute,
      st.wMonth, st.wSecond, st.wYear );

    m_access_ = ::CreateMutex( &m_sa_, true, (LPCWSTR)szUUID ); */
    m_access_ = ::CreateMutex( &m_sa_, false, 0 );

    TEAGN_TRUE_OR_THROW( ( m_access_ != 0 ),
      "Unable to create mutex object instance" )
  
  #elif TePLATFORM == TePLATFORMCODE_LINUX
  
    pthread_mutexattr_t access_attr;
    TEAGN_TRUE_OR_THROW( ( pthread_mutexattr_init( &access_attr ) == 0 ),
      "Unable to init mutex attributes" );

    pthread_mutexattr_settype( &access_attr, PTHREAD_MUTEX_DEFAULT );
    TEAGN_TRUE_OR_THROW( 
      ( pthread_mutex_init( &m_access_, &access_attr ) == 0 ),
      "Unable to init mutex" )
  
  #else
    #error "Unsuported plataform"
  #endif   
}


TeThreadLock::~TeThreadLock()
{
  unLock();
  
  #if TePLATFORM == TePLATFORMCODE_MSWINDOWS
  
    ::CloseHandle( m_access_ );
  
  #elif TePLATFORM == TePLATFORMCODE_LINUX
  
    pthread_mutex_destroy( &m_access_ );
  
  #else
    #error "Unsuported plataform"
  #endif   
}


void TeThreadLock::lock()
{
  #if TePLATFORM == TePLATFORMCODE_MSWINDOWS
  
    DWORD return_value = ::WaitForSingleObject( m_access_, INFINITE );
    TEAGN_TRUE_OR_THROW( ( ( return_value == WAIT_ABANDONED ) ||
      ( return_value == WAIT_OBJECT_0 ) ),
      "Unable to get mutex lock" );
  
  #elif TePLATFORM == TePLATFORMCODE_LINUX
  
    pthread_mutex_lock( &m_access_ );
  
  #else
    #error "Unsuported plataform"
  #endif   
}


bool TeThreadLock::tryLock()
{
  #if TePLATFORM == TePLATFORMCODE_MSWINDOWS
  
    DWORD return_value = ::WaitForSingleObject( m_access_, 
      10 );

    if( ( return_value == WAIT_OBJECT_0 ) ||
      ( return_value == WAIT_ABANDONED ) ) {

      return true;
    } else {
      return false;
    }
  
  #elif TePLATFORM == TePLATFORMCODE_LINUX
  
    if( pthread_mutex_trylock( &m_access_ ) == EBUSY ) {
      return false;
    } else {
      return true;
    }
  
  #else
    #error "Unsuported plataform"
  #endif   
}


void TeThreadLock::unLock()
{
  #if TePLATFORM == TePLATFORMCODE_MSWINDOWS
  
    ::ReleaseMutex( m_access_ );
  
  #elif TePLATFORM == TePLATFORMCODE_LINUX
  
    pthread_mutex_unlock( &m_access_ );
  
  #else
    #error "Unsuported plataform"
  #endif   
}
